@@ -7,10 +7,9 @@ Thor is a Python library for evented IO, with a focus on enabling
 high-performance HTTP intermediaries.
 """
 
-__version__ = "0.1.1"
+__version__ = "0.2"
 
 from thor.loop import run, stop, time, schedule, running
 from thor.tcp import TcpClient, TcpServer
 from thor.udp import UdpEndpoint
-from thor.http.client import HttpClient
-from thor.http.server import HttpServer
\ No newline at end of file
+from thor.events import on
\ No newline at end of file
