@@ -51,7 +51,6 @@ from thor.http.error import UrlError, ConnectError, \
 
 req_rm_hdrs = hop_by_hop_hdrs + ['host']
 
-# TODO: proxy support
 # TODO: next-hop version cache for Expect/Continue, etc.
 
 class HttpClient(object):
@@ -59,14 +58,17 @@ class HttpClient(object):
 
     tcp_client_class = TcpClient
     tls_client_class = TlsClient
-    idle_timeout = 60 # in seconds
-    connect_timeout = None
-    read_timeout = None
-    retry_limit = 2
-    retry_delay = 0.5 # in sec
 
     def __init__(self, loop=None):
         self.loop = loop or thor.loop._loop
+        self.idle_timeout = 60 # in seconds
+        self.connect_timeout = None
+        self.read_timeout = None
+        self.retry_limit = 2
+        self.retry_delay = 0.5 # in sec
+        self.proxy_tls = False
+        self.proxy_host = None
+        self.proxy_port = None
         self._conns = {}
         self.loop.on('stop', self._close_conns)
 
@@ -76,7 +78,17 @@ class HttpClient(object):
     def _attach_conn(self, origin, handle_connect,
                handle_connect_error, connect_timeout):
         "Find an idle connection for origin, or create a new one."
-        scheme, host, port = origin
+        if self.proxy_host and self.proxy_port:
+            # TODO: full form of request-target
+            import sys
+            host, port = self.proxy_host, self.proxy_port
+            if self.proxy_tls:
+                scheme = 'https'
+            else:
+                scheme = 'http'
+            origin = (scheme, host, port)
+        else:
+            scheme, host, port = origin
         while True:
             try:
                 tcp_conn = self._conns[origin].pop()
@@ -270,9 +282,9 @@ class HttpClientExchange(HttpMessageHandler, EventEmitter):
         self.output("") # kick the output buffer
         self.tcp_conn.pause(False)
 
-    def _handle_connect_error(self, err_type, err):
+    def _handle_connect_error(self, err_type, err_id, err_str):
         "The connection has failed."
-        self.input_error(ConnectError(err))
+        self.input_error(ConnectError(err_str))
 
     def _conn_closed(self):
         "The server closed the connection."
