@@ -35,10 +35,11 @@ THE SOFTWARE.
 """
 
 import errno
+import os
 import sys
 import socket
 
-from thor.loop import EventSource
+from thor.loop import EventSource, schedule
 
 
 class TcpConnection(EventSource):
@@ -105,14 +106,14 @@ class TcpConnection(EventSource):
     write_bufsize = 16
     read_bufsize = 1024 * 16
 
-    _block_errs = set([
+    _block_errs = set([(socket.error, e) for e in [
         errno.EAGAIN, errno.EWOULDBLOCK
-    ])
-    _close_errs = set([
+    ]])
+    _close_errs = set([(socket.error, e) for e in [
         errno.EBADF, errno.ECONNRESET, errno.ESHUTDOWN,
         errno.ECONNABORTED, errno.ECONNREFUSED,
         errno.ENOTCONN, errno.EPIPE
-    ])
+    ]])
 
     def __init__(self, sock, host, port, loop=None):
         EventSource.__init__(self, loop)
@@ -150,9 +151,10 @@ class TcpConnection(EventSource):
             # TODO: look into recv_into (but see python issue7827)
             data = self.socket.recv(self.read_bufsize)
         except Exception, why:
-            if why[0] in self._block_errs:
+            err = (type(why), why[0])
+            if err in self._block_errs:
                 return
-            elif why[0] in self._close_errs:
+            elif err in self._close_errs:
                 self.emit('close')
                 return
             else:
@@ -170,10 +172,11 @@ class TcpConnection(EventSource):
             data = "".join(self._write_buffer)
             try:
                 sent = self.socket.send(data)
-            except socket.error, why:
-                if why[0] in self._block_errs:
+            except Exception, why:
+                err = (type(why), why[0])
+                if err in self._block_errs:
                     return
-                elif why[0] in self._close_errs:
+                elif err in self._close_errs:
                     self.emit('close')
                     return
                 else:
@@ -251,6 +254,7 @@ class TcpServer(EventSource):
         self.sock = sock or server_listen(host, port)
         self.on('readable', self.handle_accept)
         self.register_fd(self.sock.fileno(), 'readable')
+        schedule(0, self.emit, 'start')
 
     def handle_accept(self):
         try:
@@ -269,6 +273,7 @@ class TcpServer(EventSource):
         "Stop accepting requests and close the listening socket."
         self.removeListeners('readable')
         self.sock.close()
+        self.emit('stop')
         # TODO: emit close?
 
 
@@ -289,9 +294,10 @@ class TcpClient(EventSource):
 
     Emits:
       - connect (tcp_conn): upon connection
-      - connect_error (err_type, err): if there's a problem before getting
-        a connection. err_type is socket.error or socket.gaierror; err
-        is the specific error encountered.
+      - connect_error (err_type, err_id, err_str): if there's a problem
+        before getting a connection. err_type is socket.error or
+        socket.gaierror; err_id is the specific error encountered, and
+        err_str is its textual description.
 
     To connect to a server:
 
@@ -330,19 +336,21 @@ class TcpClient(EventSource):
         try:
             err = self.sock.connect_ex((host, port))
         except socket.gaierror, why:
-            self.handle_conn_error(socket.gaierror, why[0])
+            self.handle_conn_error(socket.gaierror, why)
             return
         except socket.error, why:
-            self.handle_conn_error(socket.error, why[0])
+            self.handle_conn_error(socket.error, why)
             return
         if err != errno.EINPROGRESS:
-            self.handle_conn_error(socket.error, err)
+            self.handle_conn_error(socket.error, [err, os.strerror(err)])
             return
         if connect_timeout:
             self._timeout_ev = self._loop.schedule(
                 connect_timeout,
                 self.handle_conn_error,
-                socket.error, errno.ETIMEDOUT, True
+                socket.error,
+                [errno.ETIMEDOUT, os.strerror(errno.ETIMEDOUT)],
+                True
             )
 
     def handle_connect(self):
@@ -353,24 +361,35 @@ class TcpClient(EventSource):
             return
         err = self.sock.getsockopt(socket.SOL_SOCKET, socket.SO_ERROR)
         if err:
-            self.handle_conn_error(socket.error, err)
+            self.handle_conn_error(socket.error, [err, os.strerror(err)])
         else:
             tcp_conn = TcpConnection(
                 self.sock, self.host, self.port, self._loop
             )
             self.emit('connect', tcp_conn)
 
-    def handle_conn_error(self, err_type=None, err=None, close=False):
+    def handle_conn_error(self, err_type=None, why=None, close=False):
+        """
+        Handle a connect error.
+
+        @err_type - e.g., socket.error; defaults to socket.error
+        @why - tuple of [err_id, err_str]
+        @close - whether the error means the socket should be closed
+        """
         if self._timeout_ev:
             self._timeout_ev.delete()
         if self._error_sent:
             return
-        if err_type is None or err is None:
+        if err_type is None:
             err_type = socket.error
-            err = self.sock.getsockopt(socket.SOL_SOCKET, socket.SO_ERROR)
+            err_id = self.sock.getsockopt(socket.SOL_SOCKET, socket.SO_ERROR)
+            err_str = os.strerror(err_id)
+        else:
+            err_id = why[0]
+            err_str = why[1]
         self._error_sent = True
         self.unregister_fd()
-        self.emit('connect_error', err_type, err)
+        self.emit('connect_error', err_type, err_id, err_str)
         if close:
             self.sock.close()
 
